@@ -18,7 +18,9 @@
 #include <queue>
 
 namespace ens {
-/* After the velocity of each particle is updated at the end of each iteration
+
+/**
+ * After the velocity of each particle is updated at the end of each iteration
  * in PSO, the position of particle i (in iteration j) is updated as:
  *
  * \f[
@@ -55,6 +57,15 @@ typename MatType::elem_type PSOType<VelocityUpdatePolicy, InitPolicy>::Optimize(
       BaseMatType>();
   RequireDenseFloatingPointType<BaseMatType>();
 
+  // The number of iterations must be greater than the horizon size.
+  if (maxIterations < horizonSize)
+  {
+    std::ostringstream oss;
+    oss << "PSO::Optimize(): maxIterations (" << maxIterations << ") must be "
+        << "greater than or equal to horizonSize (" << horizonSize << ")!";
+    throw std::runtime_error(oss.str());
+  }
+
   BaseMatType& iterate = (BaseMatType&) iterateIn;
 
   // Controls early termination of the optimization process.
@@ -89,14 +100,15 @@ typename MatType::elem_type PSOType<VelocityUpdatePolicy, InitPolicy>::Optimize(
   instUpdatePolicy.As<InstUpdatePolicyType>().Initialize(exploitationFactor,
       explorationFactor, numParticles, iterate);
 
-  terminate |= Callback::BeginOptimization(*this, function, iterate,
-      callbacks...);
+  Callback::BeginOptimization(*this, function, iterate, callbacks...);
 
   // Calculate initial fitness of population.
-  for (size_t i = 0; i < numParticles; i++)
+  for (size_t i = 0; (i < numParticles) && !terminate; i++)
   {
     // Calculate fitness value.
     particleFitnesses(i) = function.Evaluate(particlePositions.slice(i));
+    terminate |= Callback::Evaluate(*this, function,
+        particlePositions.slice(i), particleFitnesses(i), callbacks...);
     particleBestFitnesses(i) = particleFitnesses(i);
   }
 
@@ -113,14 +125,16 @@ typename MatType::elem_type PSOType<VelocityUpdatePolicy, InitPolicy>::Optimize(
   // in the PSO method.
   // The performanceHorizon will be updated with the best particle
   // in a FIFO manner.
-  for (size_t i = 0; i < horizonSize; i++)
+  for (size_t i = 0; (i < horizonSize) && !terminate; i++)
   {
     // Calculate fitness and evaluate personal best.
-    for (size_t j = 0; j < numParticles; j++)
+    for (size_t j = 0; (j < numParticles) && !terminate; j++)
     {
       particleFitnesses(j) = function.Evaluate(particlePositions.slice(j));
-      Callback::Evaluate(*this, function, particlePositions.slice(j),
-          particleFitnesses(j), callbacks...);
+      terminate |= Callback::Evaluate(*this, function,
+          particlePositions.slice(j), particleFitnesses(j), callbacks...);
+      if (terminate)
+        break;
 
       // Compare and copy fitness and position to particle best.
       if (particleFitnesses(j) < particleBestFitnesses(j))
@@ -156,7 +170,7 @@ typename MatType::elem_type PSOType<VelocityUpdatePolicy, InitPolicy>::Optimize(
   }
 
   // Run the remaining iterations of PSO.
-  for (size_t i = 0; i < maxIterations - horizonSize; i++)
+  for (size_t i = 0; (i < maxIterations - horizonSize) && !terminate; i++)
   {
     // Check if there is any improvement over the horizon.
     // If there is no significant improvement, terminate.
@@ -164,11 +178,11 @@ typename MatType::elem_type PSOType<VelocityUpdatePolicy, InitPolicy>::Optimize(
       break;
 
     // Calculate fitness and evaluate personal best.
-    for (size_t j = 0; j < numParticles; j++)
+    for (size_t j = 0; (j < numParticles) && !terminate; j++)
     {
       particleFitnesses(j) = function.Evaluate(particlePositions.slice(j));
-      Callback::Evaluate(*this, function, particlePositions.slice(j),
-          particleFitnesses(j), callbacks...);
+      terminate |= Callback::Evaluate(*this, function,
+          particlePositions.slice(j), particleFitnesses(j), callbacks...);
 
       // Compare and copy fitness and position to particle best.
       if (particleFitnesses(j) < particleBestFitnesses(j))
